#!/usr/bin/python

BLOCKSIZE=1024          # Blockgroesse: 1 KB
DISKSIZE=1048576        # Plattengroesse: 1 MByte
DISKFILENAME="disk"     # Name der Image-Datei
ENTRYSIZE=32            # Groesse eines FAT-Eintrags

import mmap

# virtuelle Platte initialisieren
def init_disk():
  global disk,f
  
  # folgende vier Zeilen auskommentieren, um Inhalt der Image-Datei
  # dauerhaft zu erhalten
  disk=DISKSIZE*chr(0)              # String der Laenge DISKSIZE
  f = open (DISKFILENAME,"w")       # ... in Image-Datei schreiben
  f.write (disk)
  f.close

  f = open(DISKFILENAME, "r+")      # und Image-Datei als Memory-
  disk = mmap.mmap(f.fileno(), 0)   # Mapped-Datei oeffnen
  return

# Plattenzugriff beenden
def exit_code():
  global disk
  disk.close()                      # Memory-Mapped Image schliessen


# Block-basierte, "physikalische" Operationen auf der Platte:
# read_block()
# write_block()

# Block Nr. blocknum der Groesse BLOCKSIZE lesen
def read_block(blocknum):
  global disk
  startpos=blocknum*BLOCKSIZE
  block=disk[startpos:startpos+BLOCKSIZE]
  return block

# Block Nr. blocknum der Groesse BLOCKSIZE schreiben
def write_block(blocknum, block):
  global disk
  if len(block) != BLOCKSIZE:
    raise                           # Daten muessen richtige Laenge haben
  startpos=blocknum*BLOCKSIZE
  disk[startpos:startpos+BLOCKSIZE] = block


# Befehle fuer Verwaltung der FAT (File Allocation Table)
# fatentry_get_size()
# fat_get_entry()
# fat_find_free_entry()
# ls()
# fat_put_entry()

# Beispiel-Code, um aus FAT-Eintrag die Dateigroesse zu erhalten,
# den muessen Sie an Ihre FAT-Konstruktion anpassen.
def fatentry_get_size(entry):
  high = ord(entry[12])
  low  = ord(entry[13])
  return high*256+low

# Rueckgabe der Partitionsnummer zu einem Dateinamen
def fatentry_get_partition(entry):
  # hier ergaenzen Sie Code...
  return 0

def fat_get_entry(filename):
  fat = read_block(0)
  numberofentries = BLOCKSIZE // ENTRYSIZE
  for i in range(numberofentries):
    entry = fat[i*ENTRYSIZE:(i+1)*ENTRYSIZE]
    if entry.startswith(filename):
      print "Dateiname: ", filename
      print "FAT-Entry: ", i
      print "Groesse:   ", fatentry_get_size(entry)
      print "Part.-Nr.: ", fatentry_get_partition(entry)
  
# Funktion findet einen freien FAT-Eintrag und gibt dessen
# Nummer zurueck
def fat_find_free_entry():
  fat = read_block(0)
  numberofentries = BLOCKSIZE // ENTRYSIZE
  for i in range(numberofentries):
    entry = fat[i*ENTRYSIZE:(i+1)*ENTRYSIZE]
    if entry[0] == chr(0):
      return i
  return -1  # Error

def fs_ls():
  print "Dateiliste:"
  fat = read_block(0)
  numberofentries = BLOCKSIZE // ENTRYSIZE
  for i in range(numberofentries):
    entry = fat[i*ENTRYSIZE:(i+1)*ENTRYSIZE]
    if entry[0] != chr(0):
      filename = entry[:12]
      filesize = fatentry_get_size(entry)
      part_nr = fatentry_get_partition(entry)
      print filename, filesize, part_nr

# Neuen FAT-Eintrag erzeugen      
def fs_create_fat_entry(filename,filesize,part_nr):
  fat = read_block(0)
  pos = fat_find_free_entry()

  # Bytes 0-11: Dateiname
  # Dateiname um Blanks ergaenzen, bei Laenge 12 abschneiden  
  entry=(filename+12*" ")[:12]
  # Bytes 12, 13: Groesse (zwischen 0 und 65535)
  low  = filesize %  256    # % = modulo
  high = filesize // 256    # // = div
  entry=entry+chr(high)+chr(low)
  # Byte 14: Parititionsnummer
  entry=entry+chr(part_nr)
  # Eintrag auf maximale Laenge bringen: 32 Nullen anhaengen und dann 
  # wieder auf Laenge 32 kuerzen
  entry = entry + ENTRYSIZE*chr(0)
  entry = entry[:ENTRYSIZE]
  
  fat = fat[:pos*ENTRYSIZE] + entry + fat[(pos+1)*ENTRYSIZE:] 
  write_block(0, fat)

# In vorhandenem FAT-Eintrag Dateigroesse aktualisieren
def fs_update_fat_entry(filename,filesize):
  # das werden Sie programmieren muessen: FAT-Eintrag suchen mit
  # fat_get_entry(), veraendern und zurueck schreiben. Die Funktion 
  # fs_create_fat_entry() dient dabei als Vorlage.
  return

init_disk()

# erste Tests...
fs_create_fat_entry("test.dat",     100, 1)
fs_create_fat_entry("testdate.dat", 240, 2)

fat_get_entry("test.dat")   # sollte etwas finden
fat_get_entry("test2.dat")  # sollte nichts finden

fs_ls()

exit_code()

